@@ -1,7 +1,7 @@
 /*
  * zdev - Modify and display the persistent configuration of devices
  *
- * Copyright IBM Corp. 2016, 2017
+ * Copyright IBM Corp. 2016, 2018
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -98,26 +98,47 @@ static void dryrun_end_data(void)
 
 #define READ_CHUNK_SIZE		4096
 
-/* Read text from @fd and return resulting NULL-terminated text buffer.
- * If @chomp is non-zero, remove trailing newline character. Return %NULL
- * on error or when unprintable characters are read. */
-static char *read_fd(FILE *fd, int chomp)
+/* Read all data from @fd and return address of resulting buffer in
+ * @buffer_ptr. If @size_ptr is non-zero, use it to store the size of the
+ * resulting buffer. Return %EXIT_OK on success. */
+exit_code_t misc_read_fd(FILE *fd, void **buffer_ptr, size_t *size_ptr)
 {
 	char *buffer = NULL;
-	size_t done, i;
+	size_t done = 0;
 
-	done = 0;
 	while (!feof(fd)) {
-		buffer = realloc(buffer, done + READ_CHUNK_SIZE + 1);
+		buffer = realloc(buffer, done + READ_CHUNK_SIZE);
 		if (!buffer)
 			oom();
 		done += fread(&buffer[done], 1, READ_CHUNK_SIZE, fd);
 		if (ferror(fd)) {
 			free(buffer);
-			return NULL;
+			return EXIT_RUNTIME_ERROR;
 		}
 	}
 
+	buffer = realloc(buffer, done);
+	if (!buffer && done > 0)
+		oom();
+
+	*buffer_ptr = buffer;
+	if (size_ptr)
+		*size_ptr = done;
+
+	return EXIT_OK;
+}
+
+/* Read text from @fd and return resulting NULL-terminated text buffer.
+ * If @chomp is non-zero, remove trailing newline character. Return %NULL
+ * on error or when unprintable characters are read. */
+static char *read_fd(FILE *fd, int chomp)
+{
+	char *buffer;
+	size_t done, i;
+
+	if (misc_read_fd(fd, (void **) &buffer, &done))
+		return NULL;
+
 	/* Check if this is a text file at all (required to filter out
 	 * binary sysfs attributes). */
 	for (i = 0; i < done; i++) {
@@ -131,12 +152,13 @@ static char *read_fd(FILE *fd, int chomp)
 	if (chomp && done > 0 && buffer[done - 1] == '\n')
 		done--;
 
-	if (buffer) {
-		/* NULL-terminate. */
-		buffer[done++] = 0;
-	}
+	/* NULL-terminate. */
+	buffer = realloc(buffer, done + 1);
+	if (!buffer)
+		oom();
+	buffer[done] = 0;
 
-	return realloc(buffer, done);
+	return buffer;
 }
 
 static int count_newline(const char *str)
@@ -1069,7 +1091,7 @@ exit_code_t misc_write_text_file_retry(const char *path, const char *text,
  * at the specified path or NULL on error. */
 char *misc_readlink(const char *path)
 {
-	char *name;
+	char *name, *name2;
 	ssize_t len;
 
 	debug("Reading link %s\n", path);
@@ -1081,18 +1103,23 @@ char *misc_readlink(const char *path)
 	}
 	name[len++] = 0;
 
-	return realloc(name, len);
+	name2 = realloc(name, len);
+	return (name2) ? name2 : name;
 }
 
 /* Determine configuration set. */
-config_t get_config(int active, int persistent)
+config_t get_config(int active, int persistent, int autoconf)
 {
-	if ((!active && !persistent) || (active && persistent))
-		return config_all;
-	else if (active)
-		return config_active;
-	else
-		return config_persistent;
+	config_t config = 0;
+
+	if (active)
+		config |= config_active;
+	if (persistent)
+		config |= config_persistent;
+	if (autoconf)
+		config |= config_autoconf;
+
+	return config;
 }
 
 /* Determine if program is running under z/VM. */
@@ -1226,11 +1253,12 @@ const char *config_to_str(config_t config)
 		return "active";
 	case config_persistent:
 		return "persistent";
+	case config_autoconf:
+		return "autoconf";
 	case config_all:
 		return "all";
 	}
-	/* Should not happen. */
-	return "";
+	return "multiple";
 }
 
 /* Convert textual config representation to config_t. */
@@ -1240,6 +1268,8 @@ bool str_to_config(const char *str, config_t *config_ptr)
 		*config_ptr = config_active;
 	else if (strcasecmp(str, "persistent") == 0)
 		*config_ptr = config_persistent;
+	else if (strcasecmp(str, "autoconf") == 0)
+		*config_ptr = config_autoconf;
 	else if (strcasecmp(str, "all") == 0)
 		*config_ptr = config_all;
 	else
@@ -1688,3 +1718,18 @@ void debug_init(int argc, char *argv[])
 		fprintf(stderr, "%s\"%s\"", i > 0 ? ", " : "", argv[i]);
 	fprintf(stderr, "\n");
 }
+
+/* Return the last occurrence of @needle in @haystack, or %NULL if @needle
+ * was not found. */
+char *misc_strrstr(const char *haystack, const char *needle)
+{
+	char *result, *next;
+
+	result = strstr(haystack, needle);
+	if (result) {
+		while ((next = strstr(result + 1, needle)))
+			result = next;
+	}
+
+	return result;
+}
