@@ -20,10 +20,11 @@
 #define DISK_LAYOUT_ID			0x00000001
 
 #define ZIPL_STAGE2_LOAD_ADDRESS        0x2000
-#define ZIPL_STAGE3_ENTRY_ADDRESS	0xa050LL
+#define ZIPL_STAGE3_ENTRY_ADDRESS	0xa000LL
 #define DEFAULT_IMAGE_ADDRESS		0x10000LL
 #define KDUMP_IMAGE_ADDRESS		0x10010LL
 #define DEFAULT_STAGE3_ADDRESS          0xa000LL
+#define DEFAULT_STAGE3_PARAMS_ADDRESS	0x9000LL
 #define MINIMUM_ADDRESS 		0x10000LL
 #define ADDRESS_LIMIT 			0x80000000LL
 #define ADDRESS_LIMIT_KDUMP		0x2000000UL /* HSA size: 32 MiB */
@@ -31,11 +32,15 @@
 #define MAXIMUM_PARMLINE_SIZE 		0x380
 #define MAXIMUM_PHYSICAL_BLOCKSIZE 	0x1000
 
+#define STAGE3_HEAP_SIZE		0x4000
+#define STAGE3_HEAP_ADDRESS		0x2000
+#define STAGE3_STACK_SIZE		0x1000
+#define STAGE3_STACK_ADDRESS		0xF000
+
 #define PSW_ADDRESS_MASK		0x000000007fffffffLL
 #define PSW_LOAD                        0x0008000080000000LL
 #define PSW_DISABLED_WAIT               0x000a000000000000LL
 
-#define KERNEL_HEADER_SIZE		65536
 #define BOOTMAP_FILENAME		"bootmap"
 #define BOOTMAP_TEMPLATE_FILENAME	"bootmap_temp.XXXXXX"
 
@@ -44,12 +49,18 @@
 #define ZIPL_CONF_VAR			"ZIPLCONF"
 #define ZIPL_DEFAULT_CONF		"/etc/zipl.conf"
 #define ZIPL_DEFAULT_BLSDIR		"/boot/loader/entries"
+#define ZIPL_STAGE3_PATH		TOOLS_LIBDIR "/stage3.bin"
+#define ZIPL_SIPL_PATH			"/sys/firmware/ipl/has_secure"
 
 #define MENU_DEFAULT_PROMPT		0
 #define MENU_DEFAULT_TIMEOUT		0
 
 #define MAX_DUMP_VOLUMES		32
 
+#define SECURE_BOOT_DISABLED		0
+#define SECURE_BOOT_ENABLED		1
+#define SECURE_BOOT_AUTO		2
+
 /* Internal component load address type */
 typedef uint64_t address_t;
 
