@@ -3,7 +3,7 @@
  *
  * Functions handling the installation of the boot loader code onto disk
  *
- * Copyright IBM Corp. 2001, 2017
+ * Copyright IBM Corp. 2001, 2018
  *
  * s390-tools is free software; you can redistribute it and/or modify
  * it under the terms of the MIT license. See LICENSE for details.
@@ -23,7 +23,7 @@
 #include <syslog.h>
 #include <unistd.h>
 
-#include "lib/u2s.h"
+#include "lib/util_sys.h"
 
 #include "boot.h"
 #include "bootmap.h"
@@ -636,7 +636,7 @@ overwrite_partition_start(int fd, struct disk_info* info, int mv_dump_magic)
 	unsigned int bytes = 65536;
 	char* buffer;
 	const char dump_magic[] = {0xa8, 0x19, 0x01, 0x73,
-		0x61, 0x8f, 0x23, 0xfd};
+		0x61, 0x8f, 0x23, 0xfe};
 
 	if (misc_seek(fd, info->geo.start * info->phy_block_size))
 		return -1;
@@ -1132,7 +1132,7 @@ install_mvdump(char* const device[], struct job_target_data* target, int count,
 		parm.param[i].num_heads = info[i]->geo.heads;
 		parm.param[i].blocksize = info[i]->phy_block_size >> 8;
 		parm.param[i].devno = info[i]->devno;
-		if (u2s_getbusid(device[i], busid)) {
+		if (util_sys_get_dev_addr(device[i], busid) != 0) {
 			error_text("Could not find bus-ID for '%s'", device[i]);
 			rc = -1;
 			goto out;
