@@ -1,22 +1,23 @@
 /*
- * s390-tools/zipl/src/zipl.c
- *   zSeries Initial Program Loader tool.
+ * zipl - zSeries Initial Program Loader tool
  *
- * Copyright IBM Corp. 2001, 2009.
+ * Main function
  *
- * Author(s): Carsten Otte <cotte@de.ibm.com>
- *            Peter Oberparleiter <Peter.Oberparleiter@de.ibm.com>
+ * Copyright IBM Corp. 2001, 2017
+ *
+ * s390-tools is free software; you can redistribute it and/or modify
+ * it under the terms of the MIT license. See LICENSE for details.
  */
 
-#include "zipl.h"
-
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
 #include <errno.h>
-#include <unistd.h>
-#include <sys/types.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <unistd.h>
+
+#include "lib/zt_common.h"
 
 #include "boot.h"
 #include "bootmap.h"
@@ -25,7 +26,7 @@
 #include "install.h"
 #include "job.h"
 #include "misc.h"
-#include "zt_common.h"
+#include "zipl.h"
 
 
 /* Flag deciding the level of verbosity */
@@ -41,7 +42,7 @@ int dry_run = 1;
 static const char tool_name[] = "zipl: zSeries Initial Program Loader";
 
 /* Copyright notice */
-static const char copyright_notice[] = "Copyright IBM Corp. 2001, 2009";
+static const char copyright_notice[] = "Copyright IBM Corp. 2001, 2017";
 
 /* Usage information */
 static const char* usage_text[] = {
@@ -53,6 +54,7 @@ static const char* usage_text[] = {
 "-h, --help                      Print this help, then exit",
 "-v, --version                   Print version information, then exit",
 "-c, --config CONFIGFILE         Read configuration from CONFIGFILE",
+"-b, --blsdir BLSDIR             Parse BootLoaderSpec files from BLSDIR",
 "-t, --target TARGETDIR          Write bootmap file to TARGETDIR and install",
 "                                bootloader on device containing TARGETDIR",
 "    --targetbase BASEDEVICE     Install bootloader on BASEDEVICE",
@@ -72,8 +74,6 @@ static const char* usage_text[] = {
 "                                disk partition listed in file DEVLIST",
 "-f, --force                     Disable sanity check while producing a",
 "                                multi-volume dump",
-"-D, --dumptofs DUMPDEV[,SIZE]   Install a system dump record for dumping to",
-"                                filesystem on partition identified by DUMPDEV",
 "-m, --menu MENU                 Install multi-boot configuration MENU",
 "-n, --noninteractive            Answer all confirmation questions with 'yes'",
 "-V, --verbose                   Provide more verbose output",
@@ -88,7 +88,7 @@ print_usage(void)
 {
 	unsigned int i;
 
-	for (i=0; i < sizeof(usage_text) / sizeof(usage_text[0]); i++)
+	for (i=0; i < ARRAY_SIZE(usage_text); i++)
 		printf("%s\n", usage_text[i]);
 }
 
@@ -120,9 +120,8 @@ int
 main(int argc, char* argv[])
 {
 	struct disk_info* info;
-	disk_blockptr_t program_table;
-	disk_blockptr_t* stage2_list;
-	blocknum_t stage2_count;
+	disk_blockptr_t program_table, scsi_dump_sb_blockptr, *stage1b_list;
+	blocknum_t stage1b_count;
 	struct job_data* job;
 	char* device;
 	int rc;
@@ -143,9 +142,11 @@ main(int argc, char* argv[])
 	/* Check for priority options --help and --version */
 	if (job->id == job_print_usage) {
 		print_usage();
+		job_free(job);
 		return 0;
 	} else if (job->id == job_print_version) {
 		print_version();
+		job_free(job);
 		return 0;
 	}
 	/* Make sure we're running as root */
@@ -165,21 +166,40 @@ main(int argc, char* argv[])
 	umask(077);
 	/* Do it */
 	switch (job->id) {
+	case job_dump_partition:
+		if (disk_is_tape(job->data.dump.device) ||
+		    !disk_is_scsi(job->data.dump.device, &job->target)) {
+			rc = install_dump(job->data.dump.device, &job->target,
+					  job->data.dump.mem);
+			break;
+		}
+		/* Dump to a raw SCSI partition */
+		if (job->data.dump.mem != -1uLL) {
+			error_reason("Dump size can not be limited for "
+				     "partition dump on a SCSI disk");
+			rc = -1;
+			break;
+		}
+		rc = check_job_dump_images(&job->data.dump, job->name);
+		if (rc != 0)
+			break;
+		/* Fall through. */
 	case job_ipl:
 	case job_segment:
-	case job_dump_fs:
 	case job_menu:
 		/* Create bootmap */
-		stage2_list = NULL;
-		rc = bootmap_create(job, &program_table, &stage2_list,
-				    &stage2_count, &device, &info);
+		stage1b_list = NULL;
+		rc = bootmap_create(job, &program_table, &scsi_dump_sb_blockptr,
+				    &stage1b_list, &stage1b_count, &device,
+				    &info);
 		if (rc)
 			break;
 		/* Install boot loader */
 		rc = install_bootloader(device, &program_table,
-					stage2_list, stage2_count, info, job);
-		if (stage2_list != NULL)
-			free(stage2_list);
+					&scsi_dump_sb_blockptr, stage1b_list,
+					stage1b_count, info, job);
+		if (stage1b_list != NULL)
+			free(stage1b_list);
 		misc_free_temp_dev(device);
 		disk_free_info(info);
 		break;
@@ -192,11 +212,6 @@ main(int argc, char* argv[])
 					job->data.ipl_tape.parm_addr,
 					job->data.ipl_tape.ramdisk_addr);
 		break;
-	case job_dump_partition:
-		/* Retrieve target device information */
-		rc = install_dump(job->data.dump.device, &job->target,
-				  job->data.dump.mem);
-		break;
 	case job_mvdump:
 		rc = install_mvdump(job->data.mvdump.device,
 				    &job->target,
