@@ -35,6 +35,11 @@
 #include "misc.h"
 #include "pkey.h"
 
+/* Detect if cryptsetup 2.1 or later is available */
+#ifdef CRYPT_LOG_DEBUG_JSON
+#define HAVE_CRYPT_KEYSLOT_GET_PBKDF
+#endif
+
 #define MAX_KEY_SIZE                (8 * 1024 * 1024)
 #define MAX_PASSWORD_SIZE           512
 #define KEYFILE_BUFLEN              4096
@@ -93,6 +98,7 @@ static struct zkey_cryptsetup_globals {
 	bool inplace;
 	bool staged;
 	char *master_key_file;
+	bool batch_mode;
 	bool debug;
 	bool verbose;
 	void *lib_csulcca;
@@ -176,6 +182,11 @@ static struct util_opt opt_vec[] = {
 		.command = COMMAND_REENCIPHER,
 	},
 	OPT_PASSPHRASE_ENTRY(COMMAND_REENCIPHER),
+	{
+		.option = {"batch-mode", 0, NULL, 'q'},
+		.desc = "Suppresses all confirmation questions. Use with care!",
+		.command = COMMAND_REENCIPHER,
+	},
 	/***********************************************************/
 	{
 		.flags = UTIL_OPT_FLAG_SECTION,
@@ -204,6 +215,11 @@ static struct util_opt opt_vec[] = {
 		.command = COMMAND_SETKEY,
 	},
 	OPT_PASSPHRASE_ENTRY(COMMAND_SETKEY),
+	{
+		.option = {"batch-mode", 0, NULL, 'q'},
+		.desc = "Suppresses all confirmation questions. Use with care!",
+		.command = COMMAND_SETKEY,
+	},
 	/***********************************************************/
 	{
 		.flags = UTIL_OPT_FLAG_SECTION,
@@ -414,8 +430,13 @@ static void cryptsetup_log(int level, const char *msg,
 		fprintf(stderr, "%s: %s", program_invocation_short_name, msg);
 		break;
 	case CRYPT_LOG_DEBUG:
-		fprintf(stderr, "%s: # %s", program_invocation_short_name, msg);
+		fprintf(stderr, "%s: # %s\n", program_invocation_short_name, msg);
 		break;
+#ifdef CRYPT_DEBUG_JSON
+	case CRYPT_DEBUG_JSON:
+		fprintf(stderr, "%s\n", msg);
+		break;
+#endif
 	default:
 		warnx("Internal error on logging class for msg: %s", msg);
 		break;
@@ -1163,6 +1184,12 @@ static bool prompt_for_yes(void)
 {
 	char str[20];
 
+	if (g.batch_mode) {
+		printf("(yes implied because '--batch-mode' | '-q' option is "
+		       "specified)\n");
+		return true;
+	}
+
 	if (fgets(str, sizeof(str), stdin) == NULL)
 		return false;
 
@@ -1319,6 +1346,9 @@ static int open_keyslot(int keyslot, char **key, size_t *keysize,
 			    char **password, size_t *password_len,
 			    const char *prompt)
 {
+#ifdef HAVE_CRYPT_KEYSLOT_GET_PBKDF
+	struct crypt_pbkdf_type pbkdf;
+#endif
 	char *vkey = NULL;
 	char *pw = NULL;
 	long long tries;
@@ -1370,6 +1400,30 @@ static int open_keyslot(int keyslot, char **key, size_t *keysize,
 	keyslot = rc;
 	pr_verbose("Volume key obtained from key slot %d", keyslot);
 
+#ifdef HAVE_CRYPT_KEYSLOT_GET_PBKDF
+	/*
+	 * Get PBKDF of the key slot that was opened, and use its PBKDF for
+	 * new key slots.
+	 */
+	memset(&pbkdf, 0, sizeof(pbkdf));
+	rc = crypt_keyslot_get_pbkdf(g.cd, keyslot, &pbkdf);
+	if (rc != 0) {
+		warnx("Failed to get the PBKDF for key slot %d: %s",
+		      keyslot, strerror(-rc));
+		goto out;
+	}
+
+	/* Reuse already benchmarked number of iterations */
+	pbkdf.flags |= CRYPT_PBKDF_NO_BENCHMARK;
+
+	rc = crypt_set_pbkdf_type(g.cd, &pbkdf);
+	if (rc != 0) {
+		warnx("Failed to set the PBKDF for new key slots: %s",
+		      strerror(-rc));
+		goto out;
+	}
+#endif
+
 	if (key != NULL)
 		*key = vkey;
 	else
@@ -2199,6 +2253,9 @@ int main(int argc, char *argv[])
 		case 'm':
 			g.master_key_file = optarg;
 			break;
+		case 'q':
+			g.batch_mode = true;
+			break;
 		case 'D':
 			g.debug = true;
 			g.verbose = true;
@@ -2246,7 +2303,11 @@ int main(int argc, char *argv[])
 
 	crypt_set_log_callback(NULL, cryptsetup_log, NULL);
 	if (g.debug)
-		crypt_set_debug_level(-1);
+#ifdef CRYPT_DEBUG_JSON
+		crypt_set_debug_level(CRYPT_DEBUG_JSON);
+#else
+		crypt_set_debug_level(CRYPT_DEBUG_ALL);
+#endif
 
 	if (command->open_device) {
 		if (g.pos_arg == NULL) {
